/*******************************************************************************
 * Constants used for quick acess to Components.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Constants used in WebPanels.
 ******************************************************************************/
const nsIWebNavigation = Ci.nsIWebNavigation;
const NS_ERROR_MODULE_NETWORK = 2152398848;
const NS_NET_STATUS_READ_FROM = NS_ERROR_MODULE_NETWORK + 8;
const NS_NET_STATUS_WROTE_TO  = NS_ERROR_MODULE_NETWORK + 9;
/*******************************************************************************
 * Global Variables
 ******************************************************************************/
var gContextMenu = null;
/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var FASPanel = {
  
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _fasService: null,
  get fasService() { return this._fasService; },
  set fasService(val) { this._fasService = val; },
    
  get iframe() { return document.getElementById("fas_iframe"); },
        
  onLoad: function fsp_onLoad() {
    
    // Get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-fas", false);
    
    // Get the fas service
    this.fasService = Cc["@netscape.com/fas/service;1"].
                      getService(Ci.nsIFASService);    
          
    // Hookup the progress listener
    var docShell = this.iframe.docShell;
    var iiReq = docShell.QueryInterface(Ci.nsIInterfaceRequestor);
    var webProgress = iiReq.getInterface(Ci.nsIWebProgress);
    webProgress.addProgressListener(this, Ci.nsIWebProgress.NOTIFY_ALL);
    
    // load the url                
    this.showFAS();
  },  
  
  onUnload: function fsp_onUnload() {
    
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-fas"); 
        
    // Unhook the progress listener
    try {
      var docShell = this.iframe.docShell;
      var iiReq = docShell.QueryInterface(Ci.nsIInterfaceRequestor);
      var webProgress = iiReq.getInterface(Ci.nsIWebProgress);   
      iframe.webProgress.removeProgressListener(this);
    } catch(e) {}
 
    // check for activity
    window.parent.FAS.checkActivity();
    
    // remove variables
    this.obService = null;
    this.fasService = null;    
  },
  
  showFAS: function fsp_showFAS() {
    var url = this.fasService.URL;
    this.iframe.setAttribute("src", url);
  },
      
  // nsISupports
  QueryInterface: function fsp_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsIWebProgressListener) ||
        iid.equals(Ci.nsISupportsWeakReference) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  observe: function fsp_observe(aSubject, aTopic, aData) {
  
    // ignore other topics
    if (aTopic != "netscape-fas")
      return;

    // update
    if (aData == "signin") {   
      var self = this;
      window.setTimeout(function() { self.showFAS(); }, 0); 
    }
  },
  
  // nsIWebProgressListener    
  onProgressChange: function fsp_onProgressChange(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {},
  onLocationChange: function fsp_onLocationChange(aWebProgress, aRequest, aLocation) {},
  onStatusChange: function fsp_onStatusChange(aWebProgress, aRequest, aStatus, aMessage) {},
  onSecurityChange: function fsp_onSecurityChange(aWebProgress, aRequest, aState) {},
  onStateChange: function fsp_onStateChange(aWebProgress, aRequest, aStateFlags, aStatus) {
  
    if (!aRequest)
      return;

    // Ignore local/resource:/chrome: files
    if (aStatus == NS_NET_STATUS_READ_FROM || aStatus == NS_NET_STATUS_WROTE_TO)
      return;

    // Starting load
    if (aStateFlags & Ci.nsIWebProgressListener.STATE_START && 
        aStateFlags & Ci.nsIWebProgressListener.STATE_IS_NETWORK)
      window.parent.document.getElementById('sidebar-throbber').setAttribute("loading", "true");
    
    // Stopping load
    else if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP &&
             aStateFlags & Ci.nsIWebProgressListener.STATE_IS_NETWORK) {
      window.parent.document.getElementById('sidebar-throbber').removeAttribute("loading");
      window.parent.FAS.checkActivity();
    }      
  }
};
/*******************************************************************************
 * Context Menu object.
 ******************************************************************************/
function nsContextMenu( xulMenu ) {
  this.target            = null;
  this.menu              = null;
  this.onTextInput       = false;
  this.onLink            = false;
  this.onMailtoLink      = false;
  this.onSaveableLink    = false;
  this.link              = false;
  this.linkURL           = "";
  this.linkURI           = null;
  this.linkProtocol      = null;
  this.shouldDisplay     = false;
  this.isDesignMode      = false;
  this.docURL            = "";
  
  // Initialize new menu.
  this.initMenu( xulMenu );
}

// Prototype for nsContextMenu "class."
nsContextMenu.prototype = {
    
  // Initialize context menu.
  initMenu: function ctx_initMenu(popup) {
    // Save menu.
    this.menu = popup;

    // Get contextual info.
    this.setTarget( document.popupNode, document.popupRangeParent,
                    document.popupRangeOffset );

    // Initialize (disable/remove) menu items.
    this.initItems();
  },
    
  initItems: function ctx_initItems() {
    this.initOpenItems();
    this.initSaveItems();
    this.initNavItems();    
  },
    
  initOpenItems: function ctx_initOpenItems() {
    var shouldShow = this.onSaveableLink;
    this.showItem( "context-openlink", shouldShow);
    this.showItem( "context-openlinkintab", shouldShow);
    this.showItem( "context-openlinkinsidebar", shouldShow);
    this.showItem( "context-sep-open", shouldShow);
  },
    
  initSaveItems: function ctx_initSaveItems() {
    this.showItem( "context-savelink", this.onSaveableLink );
    this.showItem( "context-sendlink", this.onSaveableLink );
    this.showItem( "context-linkpadlink", this.onSaveableLink );
    this.showItem( "context-bookmarklink", this.onLink && !this.onMailtoLink );

    // Copy link location depends on whether we're on a link.
    this.showItem( "context-copylink", this.onLink );
  },
    
  initNavItems: function ctx_initNavItems() {
    this.showItem( "context-reload", !this.onLink);
  },
      
  // Set various context menu attributes based on the state of the world.
  setTarget: function ctx_setTarget(node, rangeParent, rangeOffset) {
    const xulNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    if ( node.namespaceURI == xulNS ) {
      return;
    }

    // Initialize contextual info.
    this.onTextInput  = false;
    this.onLink       = false;
    this.linkURL      = "";
    this.linkURI      = null;
    this.linkProtocol = "";

    // Remember the node that was clicked.
    this.target = node;
        
    // Remember the URL of the document containing the node
    // for referrer header and for security checks.
    this.docURL = node.ownerDocument.location.href;

    // Bubble out, looking for items of interest that can have childen.
    // Always pick the innermost link, background image, etc.
        
    const XMLNS = "http://www.w3.org/XML/1998/namespace";
    var elem = this.target;
    while ( elem ) {
      if ( elem.nodeType == Node.ELEMENT_NODE ) {
            
        // Link?
        if ( !this.onLink &&
           ( (elem instanceof HTMLAnchorElement && elem.href) ||
              elem instanceof HTMLAreaElement ||
              elem instanceof HTMLLinkElement ||
              elem.getAttributeNS( "http://www.w3.org/1999/xlink", "type") == "simple" ) ) {
                    
        // Target is a link or a descendant of a link.
        this.onLink = true;

        // xxxmpc: this is kind of a hack to work around a Gecko bug (see bug 266932)
        // we're going to walk up the DOM looking for a parent link node,
        // this shouldn't be necessary, but we're matching the existing behaviour for left click
        var realLink = elem;
        var parent = elem.parentNode;
        while (parent) {
          try {
            if ( (parent instanceof HTMLAnchorElement && elem.href) ||
                  parent instanceof HTMLAreaElement ||
                  parent instanceof HTMLLinkElement ||
                  parent.getAttributeNS( "http://www.w3.org/1999/xlink", "type") == "simple")
                  realLink = parent;
          } catch (e) {}
            parent = parent.parentNode;
          }
                    
          // Remember corresponding element.
          this.link = realLink;
          this.linkURL = this.getLinkURL();
          this.linkURI = this.getLinkURI();
          this.linkProtocol = this.getLinkProtocol();
          this.onMailtoLink = (this.linkProtocol == "mailto");
          this.onSaveableLink = this.isLinkSaveable( this.link );
        } 
      }
      elem = elem.parentNode;
    }
  },
  
  // Returns true if clicked-on link targets a resource that can be saved.
  isLinkSaveable: function ctx_isLinkSaveable(link) {
  
      // We don't do the Right Thing for news/snews yet, so turn them off
      // until we do.
      return this.linkProtocol && !(
             this.linkProtocol == "mailto"     ||
             this.linkProtocol == "javascript" ||
             this.linkProtocol == "news"       ||
             this.linkProtocol == "snews"      );
  },

  // Open linked-to URL in a new window.
  openLink: function ctx_openLink() {
      openNewWindowWith(this.linkURL, this.docURL, null, false);
  },
    
  // Open linked-to URL in a new tab.
  openLinkInTab: function ctx_openLinkInTab() {
      openNewTabWith(this.linkURL, this.docURL, null, null, false);
  },
    
  openLinkInSidebar: function ctx_openLinkInSidebar() {
      openWebPanel(this.linkText(), this.linkURL);
  },
       
  // Save URL of clicked-on link.
  saveLink: function ctx_saveLink() {
      urlSecurityCheck(this.linkURL, this.docURL);
      saveURL( this.linkURL, this.linkText(), null, true, false,
               makeURI(this.docURL, this.target.ownerDocument.characterSet) );
  },
    
  saveToLinkpad: function ctx_saveToLinkpad() {
      urlSecurityCheck(this.linkURL, this.docURL);
      window.parent.Linkpad.saveLink(this.linkURL, this.linkText());
  },
                        
  sendLink: function ctx_sendLink() {
    window.parent.MailIntegration.sendMessage( this.linkURL, "" ); // we don't know the title of the link so pass in an empty string
  },

  reload: function ctx_reload() {
    const reloadFlags = Ci.nsIWebNavigation.LOAD_FLAGS_NONE;
    var webNav = FASPanel.iframe.webNavigation;
    try {
      var sh = webNav.sessionHistory;
      if (sh)
        webNav = sh.QueryInterface(nsIWebNavigation);
    } catch (e) {}

    try {
      webNav.reload(reloadFlags);
    } catch (e) {}    
  },
  
  ///////////////
  // Utilities //
  ///////////////

  // Show/hide one item (specified via name or the item element itself).
  showItem: function ctx_showItem(itemOrId, show) {
    var item = itemOrId.constructor == String ? document.getElementById(itemOrId) : itemOrId;
    if (item)
      item.hidden = !show;
    if (show)
      this.shouldDisplay = true;  
  },

  // Generate fully qualified URL for clicked-on link.
  getLinkURL: function ctx_getLinkURL() {
    var href = this.link.href;
    if (href)
      return href;

    href = this.link.getAttributeNS("http://www.w3.org/1999/xlink", "href");
    if (!href || !href.match(/\S/))
      throw "Empty href"; // Without this we try to save as the current doc, for example, HTML case also throws if empty

    href = window.parent.makeURLAbsolute(this.link.baseURI, href);
    return href;
  },
    
  getLinkURI: function ctx_getLinkURI() {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    try {
      return ioService.newURI(this.linkURL, null, null);
    } catch (ex) {
      return null;
    }
  },
    
  getLinkProtocol: function ctx_getLinkProtocol() {
    if (this.linkURI)
      return this.linkURI.scheme; // can be |undefined|
    return null;
  },

  // Get text of link.
  linkText: function ctx_linkText() {
    var text = gatherTextUnder( this.link );
    if (!text || !text.match(/\S/)) {
      text = this.link.getAttribute("title");
      if (!text || !text.match(/\S/)) {
        text = this.link.getAttribute("alt");
        if (!text || !text.match(/\S/)) {
          text = this.linkURL;
        }
      }
    }
    return text;
  }
};